/**
 * Comments components.
 * @module components/theme/Comments/Comments
 */

import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { bindActionCreators } from 'redux';
import {
  FormattedMessage,
  defineMessages,
  injectIntl,
  intlShape,
} from 'react-intl';
import moment from 'moment';
import { Button, Grid, Segment } from 'semantic-ui-react';

import { addComment, deleteComment, listComments } from '../../../actions';
import { getBaseUrl } from '../../../helpers';
import { CommentEditModal, Form } from '../../../components';
import config from '../../../config';

const messages = defineMessages({
  comment: {
    id: 'Comment',
    defaultMessage: 'Comment',
  },
  commentDescription: {
    id:
      'You can add a comment by filling out the form below. Plain text formatting.',
    defaultMessage:
      'You can add a comment by filling out the form below. Plain text formatting.',
  },
  default: {
    id: 'Default',
    defaultMessage: 'Default',
  },
});

@injectIntl
@connect(
  state => ({
    items: state.comments.items,
    addRequest: state.comments.add,
    deleteRequest: state.comments.delete,
  }),
  dispatch =>
    bindActionCreators({ addComment, deleteComment, listComments }, dispatch),
)
/**
 * Comments container class.
 * @class Comments
 * @extends Component
 */
export default class Comments extends Component {
  /**
   * Property types.
   * @property {Object} propTypes Property types.
   * @static
   */
  static propTypes = {
    addComment: PropTypes.func.isRequired,
    deleteComment: PropTypes.func.isRequired,
    listComments: PropTypes.func.isRequired,
    pathname: PropTypes.string.isRequired,
    items: PropTypes.arrayOf(
      PropTypes.shape({
        author_name: PropTypes.string,
        creation_date: PropTypes.string,
        text: PropTypes.shape({
          data: PropTypes.string,
          'mime-type': PropTypes.string,
        }),
        is_deletable: PropTypes.bool,
        is_editable: PropTypes.bool,
      }),
    ).isRequired,
    addRequest: PropTypes.shape({
      loading: PropTypes.bool,
      loaded: PropTypes.bool,
    }).isRequired,
    deleteRequest: PropTypes.shape({
      loading: PropTypes.bool,
      loaded: PropTypes.bool,
    }).isRequired,
    intl: intlShape.isRequired,
  };

  /**
   * Constructor
   * @method constructor
   * @param {Object} props Component properties
   * @constructs Comments
   */
  constructor(props) {
    super(props);
    this.onSubmit = this.onSubmit.bind(this);
    this.onDelete = this.onDelete.bind(this);
    this.onEdit = this.onEdit.bind(this);
    this.onEditOk = this.onEditOk.bind(this);
    this.onEditCancel = this.onEditCancel.bind(this);
    this.state = {
      showEdit: false,
      editId: null,
      editText: null,
    };
  }

  /**
   * Component will mount
   * @method componentWillMount
   * @returns {undefined}
   */
  componentWillMount() {
    this.props.listComments(getBaseUrl(this.props.pathname));
  }

  /**
   * Component will receive props
   * @method componentWillReceiveProps
   * @param {Object} nextProps Next properties
   * @returns {undefined}
   */
  componentWillReceiveProps(nextProps) {
    if (
      nextProps.pathname !== this.props.pathname ||
      (this.props.addRequest.loading && nextProps.addRequest.loaded) ||
      (this.props.deleteRequest.loading && nextProps.deleteRequest.loaded)
    ) {
      this.props.listComments(getBaseUrl(nextProps.pathname));
    }
  }

  /**
   * Submit handler
   * @method onSubmit
   * @param {Object} formData Form data.
   * @returns {undefined}
   */
  onSubmit(formData) {
    this.props.addComment(getBaseUrl(this.props.pathname), formData.comment);
  }

  /**
   * Delete handler
   * @method onDelete
   * @param {Object} event Event object.
   * @param {string} value Delete value.
   * @returns {undefined}
   */
  onDelete(event, { value }) {
    this.props.deleteComment(value);
  }

  /**
   * Edit handler
   * @method onEdit
   * @param {Object} event Event object.
   * @param {string} value Delete value.
   * @returns {undefined}
   */
  onEdit(event, { value }) {
    this.setState({
      showEdit: true,
      editId: value.id,
      editText: value.text,
    });
  }

  /**
   * On edit ok
   * @method onEditOk
   * @returns {undefined}
   */
  onEditOk() {
    this.setState({
      showEdit: false,
      editId: null,
      editText: null,
    });
    this.props.listComments(getBaseUrl(this.props.pathname));
  }

  /**
   * On edit cancel
   * @method onEditCancel
   * @returns {undefined}
   */
  onEditCancel() {
    this.setState({
      showEdit: false,
      editId: null,
      editText: null,
    });
  }

  /**
   * Render method.
   * @method render
   * @returns {string} Markup for the component.
   */
  render() {
    return (
      <div className="comments">
        <CommentEditModal
          open={this.state.showEdit}
          onCancel={this.onEditCancel}
          onOk={this.onEditOk}
          id={this.state.editId}
          text={this.state.editText}
        />
        {this.props.items.map(item => [
          <div className="comment" key={item['@id']}>
            <Grid stackable>
              <Grid.Column width={6}>
                <FormattedMessage
                  id="{user} says:"
                  defaultMessage="{user} says:"
                  values={{
                    user: <span>{item.author_name}</span>,
                  }}
                />
              </Grid.Column>
              <Grid.Column width={6} textAlign="right">
                <span title={moment(item.creation_date).format('LLLL')}>
                  {moment(item.creation_date).fromNow()}
                </span>
              </Grid.Column>
            </Grid>
            <Segment clearing>
              {item.text.data}
              {item.is_deletable && (
                <Button
                  onClick={this.onDelete}
                  value={item['@id'].replace(config.apiPath, '')}
                  color="red"
                  floated="right"
                >
                  <FormattedMessage id="Delete" defaultMessage="Delete" />
                </Button>
              )}
              {item.is_editable && (
                <Button
                  onClick={this.onEdit}
                  floated="right"
                  value={{
                    id: item['@id'].replace(config.apiPath, ''),
                    text: item.text.data,
                  }}
                >
                  <FormattedMessage id="Edit" defaultMessage="Edit" />
                </Button>
              )}
            </Segment>
            <br />
          </div>,
        ])}
        <Form
          onSubmit={this.onSubmit}
          submitLabel={this.props.intl.formatMessage(messages.comment)}
          resetAfterSubmit
          title={this.props.intl.formatMessage(messages.comment)}
          schema={{
            fieldsets: [
              {
                fields: ['comment'],
                id: 'default',
                title: this.props.intl.formatMessage(messages.default),
              },
            ],
            properties: {
              comment: {
                title: this.props.intl.formatMessage(messages.comment),
                type: 'string',
                widget: 'textarea',
              },
            },
            required: ['comment'],
          }}
        />
      </div>
    );
  }
}
